@@ -1,4 +1,3 @@
-import json
 import math
 from typing import List, Optional
 
@@ -121,6 +120,14 @@ class Document:
             for fact in facts:
                 self.add_fact(fact)
 
+    @staticmethod
+    def remove_duplicates_with_ignored_keys(duplicates, ignore_keys):
+        seen = set()
+        for duplicate in duplicates:
+            index = frozenset((k, v) for k, v in duplicate.items() if k not in ignore_keys)
+            if index not in seen:
+                yield duplicate
+                seen.add(index)
 
     @staticmethod
     def handle_null_values_in_facts(facts: List[dict]):
@@ -136,13 +143,11 @@ class Document:
     def remove_duplicate_facts(facts: List[dict]):
         if facts:
             facts = Document.handle_null_values_in_facts(facts)
-            set_of_jsons = {json.dumps(fact, sort_keys=True, ensure_ascii=False) for fact in facts}
-            without_duplicates = [json.loads(unique_fact) for unique_fact in set_of_jsons]
+            without_duplicates = list(Document.remove_duplicates_with_ignored_keys(facts, ["id", "source"]))
             return without_duplicates
         else:
             return []
 
-
     def facts_to_json(self) -> dict:
         facts = [fact.to_json() for fact in self.__texta_facts]
         unique_facts = Document.remove_duplicate_facts(facts)
@@ -259,9 +264,16 @@ class Document:
         return {"text_mlp": container, **texta_facts}
 
 
+    @staticmethod
+    def _clean_lemma(lemma):
+        if len(lemma) > 2:
+            return lemma.replace("_", "").replace("=", "")
+        return lemma
+
+
     def lemmas(self):
         for sent in self.stanza_sentences:
-            self.__lemmas.append([word.lemma.replace("_", "") if word and word.lemma else "X" for word in sent.words])
+            self.__lemmas.append([self._clean_lemma(word.lemma) if word and word.lemma else "X" for word in sent.words])
 
 
     def get_lemma(self) -> str:
@@ -340,6 +352,7 @@ class Document:
         for entity_type, entity_values in hits.items():
             for entity_value in entity_values:
                 new_fact = Fact(
+                    source="mlp",
                     fact_type=entity_type,
                     fact_value=entity_value["value"],
                     doc_path=self.__get_doc_path("text"),
@@ -350,6 +363,7 @@ class Document:
         for entity_type, entity_values in lemma_hits.items():
             for entity_value in entity_values:
                 new_fact = Fact(
+                    source="mlp",
                     fact_type=entity_type,
                     fact_value=entity_value["value"],
                     doc_path=self.__get_doc_path("lemmas"),
@@ -383,6 +397,7 @@ class Document:
                     # recheck that string contains a number
                     if any(map(str.isdigit, fact_value)):
                         new_fact = Fact(
+                            source="mlp",
                             fact_type="CURRENCY_SUM",
                             fact_value=fact_value,
                             doc_path=self.__get_doc_path("text"),
@@ -502,6 +517,7 @@ class Document:
                 text_before_match = tokenized_text[:best_matching_span[0]]
                 sentence_index = text_before_match.count("\n")
                 new_fact = Fact(
+                    source="mlp",
                     fact_type=entity.type,
                     fact_value=entity.text,
                     doc_path=self.__get_doc_path("text"),
