@@ -1,10 +1,13 @@
 import json
+import uuid
 from typing import List
 
 
 class Fact:
 
-    def __init__(self, fact_type: str, fact_value: str, doc_path: str = None, fact_lemma: str = None, spans: list = [[0, 0]], sent_index: int = 0):
+    def __init__(self, source: str, fact_type: str, fact_value: str, doc_path: str = None, fact_lemma: str = None, spans: list = [[0, 0]], sent_index: int = 0):
+        self.id = str(uuid.uuid4())
+        self.source = source
         self.fact_type = fact_type
         self.fact_value = fact_value
         self.spans = spans
@@ -22,13 +25,15 @@ class Fact:
             sent_index = fact_json.get("sent_index")
             value = fact_json.get("str_val")
             field_path = fact_json.get("doc_path")
-            fact = Fact(fact_type=fact_type, fact_value=value, doc_path=field_path, spans=spans)
+            fact = Fact(source="mlp", fact_type=fact_type, fact_value=value, doc_path=field_path, spans=spans)
             container.append(fact)
         return container
 
 
     def to_json(self):
         container = {
+            "id": self.id,
+            "source": "mlp",
             "str_val": self.fact_value,
             "spans": json.dumps(self.spans),
             "sent_index": self.sent_index,
